#! /usr/bin/python2.3
""" bank12.py: Simulate customers arriving
    at random, using a Source requesting service
    from two clerks but a single queue
    with a random servicetime
    Uses a Monitor object to record waiting times
    Set up 4 replications 
""" 
from __future__ import generators
from SimPy.Simulation  import *
from SimPy.Monitor import *
from random import Random

class Source(Process):
    """ Source generates customers randomly"""
    def __init__(self,seed=333):
        Process.__init__(self)
        self.SEED = seed

    def generate(self,number,interval):       
        rv = Random(self.SEED)
        for i in range(number):
            c = Customer(name = "Customer%02d"%(i,))
            activate(c,c.visit(timeInBank=12.0))
            t = rv.expovariate(1.0/interval)
            yield hold,self,t

class Customer(Process):
    """ Customer arrives, is served and leaves """
    def __init__(self,name):
        Process.__init__(self)
        self.name = name
        
    def visit(self,timeInBank=0):       
        arrive=now()
        ##print "%7.4f %s: Here I am "%(now(),self.name)
        yield request,self,counter
        wait=now()-arrive
        waitMonitor.tally(wait)
        ##print "%7.4f %s: Waited %6.3f"%(now(),self.name,wait)
        tib = counterRV.expovariate(1.0/timeInBank)
        yield hold,self,tib
        yield release,self,counter
        ##print "%7.4f %s: Finished"%(now(),self.name)

def model(counterseed=3939393):
    global counter,counterRV,waitMonitor
    counter = Resource(name="Clerk",capacity = 2)
    counterRV = Random(counterseed)
    waitMonitor = Monitor()
    initialize()
    sourceseed=1133
    source = Source(seed = sourceseed)
    activate(source,source.generate(50,10.0),0.0)
    simulate(until=2000.0)
    return waitMonitor.mean()

result = [model(counterseed=cs) for cs in [13939393,31555999,777999555,319999771]] #Lmula
for x in result: print "meanwait = %7.4f"%(x,)                                     #Lmulb
